import java.util.*;
import java.util.regex.*;
import java.net.*;
import java.io.*;
import java.awt.*;

//possible status values are:
//    "STATUS_SOLITARY_CONFINEMENT", "STATUS_JAIL_CONJUGAL_VISITS", "STATUS_NONE", and "STATUS_LETHAL_INJECTION"

//possible times are:
// 0 (red-wait), 1 (blue-register), 2 (green-can_alarm), 3 (yellow-can_alarm)
public class PeopleDB extends Frame{
	private Hashtable DB;
	private Vector Names;
	// names are hard coded and must match with names in ControlMonitor.java
	private String names[] = {"Shivashankar Balu", "Ding Liu",           "Ross Messing",
	                        "Matthew Post",      "Arrvindh Shriraman", "Jacob Balazar",
	                        "Piotr Faliszewski", "Michael Spear",      "Wenzhao Tan",
	                        "Anustup Choudhury", "Pin Lu",             "Tanushree Mittal",
	                        "Lior Privman",      "Benjamin VanDurme"};

	//1-to-1 correspondance with names :)
	private String teams[] = {"red", "red", "red", "red", "red",
	                          "green", "green", "green", "green",
	                          "blue", "blue", "blue", "blue", "blue"};
	
	private TextArea display, timedisplay;
	private DBTimeListenerClient timeclient;
	public int time;
	private String timeServer;
	
	public PeopleDB(String t){
		super("Database");
		// create areas for writing the database and time
		display = new TextArea("", 0, 0, TextArea.SCROLLBARS_VERTICAL_ONLY);
		add (display, BorderLayout.CENTER);
		timedisplay = new TextArea("",0,0, TextArea.SCROLLBARS_VERTICAL_ONLY);
		add (timedisplay, BorderLayout.SOUTH);
		
		// initialize time
		timeServer=t;
		time=0;
		timeclient = new DBTimeListenerClient(this, timeServer);
		timeclient.start();
		
		//create the names database
		Names = new Vector();
		DB = new Hashtable();
		for (int i=0;i<names.length;i++)
			DB.put(names[i], new PersonInfo(names[i], teams[i]));
		
		//show window
		setSize(450, 550);
		setVisible(true);
	}
	
	// debug use only?
	// put string onto the timedisplay
	public void MsgToTimeDisplay(String s){
		timedisplay.append(s);
	}
	
	// move people out of status (to include jail) on clock tick
	public void Cycle_Update(){
		String n;
		int i;
		PersonInfo p;
		
		for (i=0;i<names.length;i++){
			//get each person
			n = names[i];
			p = (PersonInfo) DB.get(n);
			//update jail status
			if (p.status.equals("STATUS_JAIL_CONJUGAL_VISITS")) p.status = "STATUS_NONE";
			else if (p.status.equals("STATUS_SOLITARY_CONFINEMENT")) p.status = "STATUS_JAIL_CONJUGAL_VISITS";
			else if (p.status.equals("STATUS_NEWJAIL")) p.status = "STATUS_SOLITARY_CONFINEMENT";
			else if (!p.status.equals("STATUS_LETHAL_INJECTION")) p.status = "STATUS_NONE"; //default case is to reset to none
			//re-insert
			DB.put(n, p);
		}
		DisplayDB();
	}
	
	//print the whole table of people
	private void DisplayDB(){
		display.replaceRange("", 0,10000);
		PersonInfo p;
		String n;
		for (int i=0;i<names.length;i++){
			n = names[i];
			p = (PersonInfo) DB.get(n);
			display.append(p.name+":"+p.team+":"+p.status+":"+p.strikes+"\n");
		}
	}
	
	//update a location if the status is "none"
	private void SetLocation(String name, String location){
		if (time==1){//only register during blue period
			PersonInfo p;
			p = (PersonInfo) DB.get(name);
			if (p.status.equals("STATUS_NONE")){
				p.status = location;
				DB.put(name,p);
			}
		}
	}
	
	// register an arrest
	private void RegisterArrest(String name){
		if (time==2||time==3){//only alarm during green or yellow period
			PersonInfo p;
			p = (PersonInfo) DB.get(name);
			//can't arrest the arrested...
			if (!p.status.equals("STATUS_LETHAL_INJECTION")&&!p.status.equals("STATUS_CONJUGAL_VISITS")&&!p.status.equals("STATUS_SOLITARY_CONFINEMENT")){
					// update the # of strikes
				p.strikes++;
				if(p.strikes==3)
					p.status="STATUS_LETHAL_INJECTION";
				else
					p.status="STATUS_NEWJAIL";
				DB.put(name, p);
			}
		}
	}
	
	//networking code:  listen for arrests and registrations
	//executives should interface to this loop...
	private void ListenPort(){
		ServerSocket server;
		Socket connection;
		DataOutputStream output;
		DataInputStream input;
		String inputMsg;
		String msg[];
		
		try{
			server = new ServerSocket(9999);
			while (true){
				//accept a connection
				connection = server.accept();
				//get io streams from connection
				input = new DataInputStream(connection.getInputStream());
				output = new DataOutputStream(connection.getOutputStream());
				//read the message
				inputMsg = input.readUTF();
				//parse the message
				msg = inputMsg.split(":");
				if (msg[0].equals("entered")&&msg.length==3)
					SetLocation(msg[2], msg[1]);
				if (msg[0].equals("arrested")&&msg.length==2)
					RegisterArrest(msg[1]);
				connection.close();
				//update database
				DisplayDB();
			}
		}
		catch (IOException e){
			e.printStackTrace();
		}
	}
	
	//main function
	public static void main (String args[]){
		if (args.length!=1){
			System.out.println("ERROR: you must specify the name of the time server\n");
			System.out.println("Proper usage:  java PeopleDB time.cs.rochester.edu");
			System.out.println("Aborting");
			System.exit(1);
		}
		else{
			PeopleDB db = new PeopleDB(args[0]);
			db.addWindowListener(new CloseWindow());
			db.ListenPort();
		}
	}
}

//thread to listen for clock ticks
class DBTimeListenerClient extends Thread{
	String timeServer;
	PeopleDB parent;
	
	public DBTimeListenerClient(PeopleDB p, String s){
		super();
		timeServer = s;
		parent=p;
	}
	
	public void run(){
		String tick;
		String tock="";//last color
		String msg[];
		try{
	    //9990 is where we listen for ticks
			ServerSocket servSock = new ServerSocket(9990);
	    //register thread with the time server so he knows to send to us
			Socket       sock= new Socket(timeServer,9991);
	    //registration done
			sock.close();

	    while(true){
				//get connection
				sock = servSock.accept();
				//read tick
				tick = (new DataInputStream(sock.getInputStream())).readUTF();
				msg = tick.split(":");
				if (msg.length>3)
					//conditions to see when we need to run cycle_update
					//i.e., was it NOT RED and now it IS RED?
					if (!msg[2].equals(tock))
						if (msg[2].equals("0")) //0=red
							parent.Cycle_Update();
				//push the time to the parent
				tock = msg[2];
				parent.time=new Integer(tock).intValue();
				//tell parent to print time
				parent.MsgToTimeDisplay(tick);
				sock.close();
	    }
		}
		catch( Exception e ){
	    e.printStackTrace();
		}
	}
}

// don't you wish java had a struct type like C?  then this wouldn't be necessary
class PersonInfo{
	public String name;
	public String team;
	public String status; // can be STATUS_IN<color>
	public int strikes;
	
	public PersonInfo(String n, String t){
		name = n;
		team = t;
		status="STATUS_NONE"; //default is no status
		strikes=0; // default is no strikes
	}
}
